// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses
  System.Collections.Generic,
  Esmf,
  Fund.CommonDimensions;

type
  OptimalTaxation = public class
  private
    _prtp: Double := 0.001;
    _eta: Double := 1.0;
    _useNegishiWeights: Boolean := true;
    _optiterations: Integer := 5;
  protected
  public
    method Run;
    method GetFullSCCValues(taxes: IDictionary<KeyValuePair<Timestep,Region>,double>): IDictionary<KeyValuePair<Timestep,Region>,double>;
    
    property RunData: Run read write;
    property Parameters: FundParameter read write;
    property Prtp: Double read _prtp write _prtp;
    property Eta: Double read _eta write _eta; 
    property UseNegishiWeights: Boolean read _useNegishiWeights write _useNegishiWeights;     
    property PostRunMethod: Action<FundWorkflow> read write; 
    property NumberOfOptimizationIterations: Integer read _optiterations write _optiterations;   
  end;
  
implementation

method OptimalTaxation.Run; 
begin    
  var x:=GetFullSCCValues(nil);
  
  for i: Integer :=0 to NumberOfOptimizationIterations do
  begin
    Console.WriteLine('Iteration {0}', i+1);
    var y:=GetFullSCCValues(x);
    x:=y;
  end;
  
  // Do final run with optimal taxation for data output
  var fundModel:=new FundWorkflow(Parameters);
  
  fundModel.ModelFields.AttachedDimensions.Add(UseNegishiWeights.ToString());
  fundModel.ModelFields.AttachedDimensions.Add(eta.ToString("f1"));
  fundModel.ModelFields.AttachedDimensions.Add(prtp.ToString("f3"));
  
  fundModel.ModelFields.SetNonDimensionalFieldValue('socioeconomic','runwithoutpopulationperturbation', true);
  fundModel.ModelFields.SetNonDimensionalFieldValue('socioeconomic','consleak', 0.0);    
  fundModel.ModelFields.Set2DimensionalParameter<Timestep,Region,Double>('socioeconomic','currtax',
    method(t: Timestep; r: Region): Double;
    begin
      if t<Timestep.FromYear(2005) then
        Result:=0.0
      else
        Result:=x[new KeyValuePair<Timestep,Region>(t,r)];
    end);
  
  fundmodel.Run(nil);
    
  if PostRunMethod<>nil then
    PostRunMethod(fundModel);

  Console.WriteLine('Done');
end;

method OptimalTaxation.GetFullSCCValues(taxes: IDictionary<KeyValuePair<Timestep,Region>,double>): IDictionary<KeyValuePair<Timestep,Region>,double>; 
begin
  var sccs:=new Dictionary<KeyValuePair<Timestep,Region>,double>;
  var clock:=new Clock(Timestep.FromYear(2005),Timestep.FromSimulationYear(LegacyConstants.NYear));
  
  while not clock.IsDone do
  begin
    
    var lMarginalRun2:=new MarginalDamage2(
      Run:=RunData,
      Gas:=MarginalGas.C,
      EmissionYear:=clock.Current,
      Prtp:=Prtp,
      Eta:=Eta,
      UseNegishiWeights:=UseNegishiWeights,
      Parameters:=Parameters);
      
    if taxes<>nil then
      lMarginalRun2.AdditionalInitMethod:=
        method(f: FundWorkflow);
        begin
          f.ModelFields.Set2DimensionalParameter<Timestep,Region,Double>('socioeconomic','currtax',
          method(t: Timestep; r: Region): Double;
          begin
            if t<Timestep.FromYear(2005) then
              Result:=0.0
            else
              Result:=taxes[new KeyValuePair<Timestep,Region>(t,r)];
          end);
        end;
      
    var scc:=lMarginalRun2.Start;
    
    for each r: Region in Enum.GetValues(typeof(Region)) do
    begin
      sccs.Add(new KeyValuePair<Timestep,Region>(clock.Current,r),scc[r]);    
    end;
    
    Console.WriteLine('{0}, USA SCC: {1:f2}, USA tax: {3:f2}, CHI SCC: {2:f2}, CHI tax: {4:f2}', 
      clock.Current, 
      scc[Region.USA], 
      scc[Region.CHI],
      iif(taxes<>nil,taxes[new KeyValuePair<Timestep,Region>(clock.Current,Region.USA)],0.0),
      iif(taxes<>nil,taxes[new KeyValuePair<Timestep,Region>(clock.Current,Region.CHI)],0.0));
    clock.Advance;
  end;
  
  Result:=sccs;
end;

end.
