// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses
  System.Collections;

type

  // Abstract base class for a specific weighting scheme
  // To implement a specific weighting scheme, derive from this
  // class
  WeightingScheme = public abstract class 
  private
    m_weights: array [0..LegacyConstants.NYear-1, 0..LegacyConstants.NoReg-1] of double;
  protected
    // Return the name of the weighting scheme
    // The name is going to be used by output modules
    function GetName: string; virtual; abstract;

    // Return the weight for the specified year and region
    function GetWeight(Year: Integer; Region: integer): double; virtual;

    procedure AddWeight(Year: Integer; Region: integer; WeightData: double);
  public
    constructor;

    procedure CalculateWeights(ModelOutput: ModelOutput); virtual; abstract;
    
    property Name: string read GetName;
    property Weight[Year: Integer; Region: integer]: double read GetWeight; default;
  end;

  WeightingCombination = class
  private
    m_weightingSchemes: ArrayList;
    m_name: String;
  protected
    function GetName: string;
    function GetWeight(Year: Integer; Region: integer): double; virtual;
  public
    constructor;
    constructor(name: String);

    procedure Add(WeightingScheme: WeightingScheme);
    procedure CalculateWeights(ModelOutput: ModelOutput);

    function AddDamagesUp(Damages: Damages; yearsToAggregate: integer; emissionPeriod: Integer): double;

    property Name: string read GetName;
    property Weight[Year: Integer; Region: integer]: double read GetWeight; default;
  end;

implementation

procedure WeightingScheme.AddWeight(Year: Integer; Region: integer;
  WeightData: double);
begin
  m_weights[Year-1, Region-1]:=WeightData;
end;

constructor WeightingScheme;
begin
  inherited constructor;

end;

function WeightingScheme.GetWeight(Year: Integer;
  Region: integer): double;
begin
  Result:=m_weights[Year-1, Region-1];
end;

{ WeightingCombination }

procedure WeightingCombination.Add(WeightingScheme: WeightingScheme);
begin
  m_weightingSchemes.Add(WeightingScheme);
end;

function WeightingCombination.AddDamagesUp(Damages: Damages; yearsToAggregate: integer; emissionPeriod: Integer): double;
var
  i: integer;
  i_totalDamage: double;
  i_Damage: Damage;
begin
  i_totalDamage:=0;
  for i:=0 to Damages.Count - 1 do
  begin
    i_Damage:=Damages.Item[i];
    if (i_Damage.Year >= emissionPeriod-1950) AND (i_Damage.Year < emissionPeriod-1950 + yearsToAggregate) then
      i_totalDamage:=i_totalDamage + i_Damage.Damage * Weight[i_Damage.Year, i_Damage.Region];
  end;
  Result:=i_totalDamage;
end;

procedure WeightingCombination.CalculateWeights(ModelOutput: ModelOutput);
var
  i: Integer;
begin
  for i:=0 to m_weightingSchemes.Count - 1 do
    WeightingScheme(m_weightingSchemes[i]).CalculateWeights(ModelOutput);
end;

constructor WeightingCombination;
begin
  inherited constructor;
  m_weightingSchemes:=new ArrayList;
  m_name:=nil;
end;

constructor WeightingCombination(name: String);
begin
  inherited constructor;
  m_weightingSchemes:=new ArrayList;
  m_name:=name;
end;

function WeightingCombination.GetName: string;
var
  i: integer;
  i_name: string;
begin
  if m_name=nil then
  begin
    if m_weightingSchemes.Count > 0 then
    begin
      i_name:='';
      for i:=0 to m_weightingSchemes.Count - 1 do
      begin
        if i>0 then
          i_name:=i_name + ' + ';
        i_name:=i_name + WeightingScheme(m_weightingSchemes[i]).Name;
      end;
    end
    else
      i_name:='none';
  end
  else
    i_name:=m_name;
    
  Result:=i_name;
end;

function WeightingCombination.GetWeight(Year: Integer;
  Region: integer): double;
var
  i: integer;
  i_weight: double;
begin
  i_weight:=1;
  for i:=0 to m_weightingSchemes.Count - 1 do
    i_weight:=i_weight * WeightingScheme(m_weightingSchemes[i]).Weight[Year, Region];
  Result:=i_weight;
end;

end.